@@ -18,6 +18,10 @@
      SPDX-License-Identifier: AGPL3.0-or-later
  */
 
+#if ! defined (__GNUNET_UTIL_LIB_H_INSIDE__)
+#error "Only <gnunet_util_lib.h> can be included directly."
+#endif
+
 /**
  * @addtogroup libgnunetutil
  * Multi-function utilities library for GNUnet programs
@@ -45,8 +49,7 @@ struct GNUNET_ChildWaitHandle;
  * @param cls handle for the callback
  * @param type type of the process
  * @param exit_code status code of the process
- *
-*/
+ */
 typedef void
 (*GNUNET_ChildCompletedCallback)(void *cls,
                                  enum GNUNET_OS_ProcessStatusType type,
