@@ -27,7 +27,7 @@
  * @file
  * Monitoring / diagnostics API for the transport service
  *
- * @defgroup transport TRANSPORT service
+ * @defgroup TNG Transport Next Generation service
  * Communication with other peers
  *
  * @see [Documentation](https://gnunet.org/transport-service)
@@ -45,8 +45,8 @@ extern "C"
 #endif
 #endif
 
+
 #include "gnunet_util_lib.h"
-#include "gnunet_ats_transport_service.h"
 #include "gnunet_transport_communication_service.h"
 
 
