@@ -1244,6 +1244,62 @@ GNUNET_CONFIGURATION_serialize (const struct GNUNET_CONFIGURATION_Handle *cfg,
 }
 
 
+/**
+ * Dump an os installation path
+ * to a buffer.
+ *
+ * @param cfg configuration
+ * @param buf buffer to write to
+ * @param ipk which path to print
+ */
+static void
+dump_os_ipk (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  struct GNUNET_Buffer *buf,
+  enum GNUNET_OS_InstallationPathKind ipk)
+{
+  char *v = GNUNET_OS_installation_get_path (cfg->pd,
+                                             ipk);
+  const char *ipkname;
+  switch (ipk)
+  {
+  case GNUNET_OS_IPK_DATADIR:
+    ipkname = "IPK_DATADIR";
+    break;
+  case GNUNET_OS_IPK_BINDIR:
+    ipkname = "IPK_BINDIR";
+    break;
+  case GNUNET_OS_IPK_LIBDIR:
+    ipkname = "IPK_LIBDIR";
+    break;
+  case GNUNET_OS_IPK_PREFIX:
+    ipkname = "IPK_PREFIX";
+  case GNUNET_OS_IPK_LOCALEDIR:
+    ipkname = "IPK_LOCALEDIR";
+    break;
+  case GNUNET_OS_IPK_ICONDIR:
+    ipkname = "IPK_ICONDIR";
+  case GNUNET_OS_IPK_DOCDIR:
+    ipkname = "IPK_DOCDIR";
+    break;
+  case GNUNET_OS_IPK_LIBEXECDIR:
+    ipkname = "IPK_LIBEXECDIR";
+    break;
+  case GNUNET_OS_IPK_SELF_PREFIX:
+    ipkname = "IPK_SELF_PREFIX";
+    break;
+  default:
+    ipkname = "??";
+    break;
+  }
+  GNUNET_buffer_write_fstr (buf,
+                            "# %s = %s\n",
+                            ipkname,
+                            v);
+  GNUNET_free (v);
+}
+
+
 char *
 GNUNET_CONFIGURATION_serialize_diagnostics (
   const struct GNUNET_CONFIGURATION_Handle *cfg)
@@ -1288,6 +1344,20 @@ GNUNET_CONFIGURATION_serialize_diagnostics (
                              "\n");
   }
 
+  GNUNET_buffer_write_fstr (&buf,
+                            "#\n# Installation paths:\n");
+
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_DATADIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_LIBDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_BINDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_PREFIX);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_LOCALEDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_ICONDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_DOCDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_LIBEXECDIR);
+  dump_os_ipk (cfg, &buf, GNUNET_OS_IPK_SELF_PREFIX);
+
+
   GNUNET_buffer_write_fstr (&buf,
                             "#\n\n");
 
@@ -1711,23 +1781,24 @@ GNUNET_CONFIGURATION_get_value_number (
   return GNUNET_OK;
 }
 
+
 void
 GNUNET_CONFIGURATION_set_value_float (struct GNUNET_CONFIGURATION_Handle *cfg,
-                                       const char *section,
-                                       const char *option,
-                                       float number)
+                                      const char *section,
+                                      const char *option,
+                                      float number)
 {
   char s[64];
 
   // TODO FIXME note that this truncates the float
   // #9369
-  const locale_t cl = newlocale(LC_NUMERIC_MASK, "C", (locale_t)0);
-  locale_t old_locale = uselocale(cl);
+  const locale_t cl = newlocale (LC_NUMERIC_MASK, "C", (locale_t) 0);
+  locale_t old_locale = uselocale (cl);
   GNUNET_snprintf (s,
                    64,
                    "%f",
                    number);
-  uselocale(old_locale);
+  uselocale (old_locale);
   GNUNET_CONFIGURATION_set_value_string (cfg,
                                          section,
                                          option,
@@ -1752,14 +1823,14 @@ GNUNET_CONFIGURATION_get_value_float (
   if (NULL == e->val)
     return GNUNET_NO;
   // #9369
-  const locale_t cl = newlocale(LC_NUMERIC_MASK, "C", (locale_t)0);
-  locale_t old_locale = uselocale(cl);
+  const locale_t cl = newlocale (LC_NUMERIC_MASK, "C", (locale_t) 0);
+  locale_t old_locale = uselocale (cl);
   if (1 != sscanf (e->val,
                    "%f%1s",
                    number,
                    dummy))
     return GNUNET_SYSERR;
-  uselocale(old_locale);
+  uselocale (old_locale);
   return GNUNET_OK;
 }
 
