@@ -31,10 +31,10 @@
 #define LOG(kind, ...) GNUNET_log_from (kind, "util-disk", __VA_ARGS__)
 
 #define LOG_STRERROR(kind, syscall) \
-  GNUNET_log_from_strerror (kind, "util-disk", syscall)
+        GNUNET_log_from_strerror (kind, "util-disk", syscall)
 
 #define LOG_STRERROR_FILE(kind, syscall, filename) \
-  GNUNET_log_from_strerror_file (kind, "util-disk", syscall, filename)
+        GNUNET_log_from_strerror_file (kind, "util-disk", syscall, filename)
 
 /**
  * Block size for IO for copying files.
@@ -58,7 +58,7 @@
 #ifndef S_ISLNK
 #define _IFMT 0170000 /* type of file */
 #define _IFLNK 0120000 /* symbolic link */
-#define S_ISLNK(m) (((m) & _IFMT) == _IFLNK)
+#define S_ISLNK(m) (((m)&_IFMT) == _IFLNK)
 #endif
 
 
@@ -151,17 +151,17 @@ get_size_rec (void *cls, const char *fn)
   ! (defined(_FILE_OFFSET_BITS) && _FILE_OFFSET_BITS == 64)
   struct stat64 buf;
 
-  if (0 != stat64 (fn, &buf))
+  if (0 != lstat64 (fn, &buf))
   {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_DEBUG, "stat64", fn);
+    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_DEBUG, "lstat64", fn);
     return GNUNET_SYSERR;
   }
 #else
   struct stat buf;
 
-  if (0 != stat (fn, &buf))
+  if (0 != lstat (fn, &buf))
   {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_DEBUG, "stat", fn);
+    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_DEBUG, "lstat", fn);
     return GNUNET_SYSERR;
   }
 #endif
@@ -356,7 +356,7 @@ GNUNET_DISK_mkdtemp (const char *t)
 }
 
 
-void
+char *
 GNUNET_DISK_file_backup (const char *fil)
 {
   size_t slen;
@@ -366,14 +366,51 @@ GNUNET_DISK_file_backup (const char *fil)
   slen = strlen (fil) + 20;
   target = GNUNET_malloc (slen);
   num = 0;
+
+#if HAVE_RENAMEAT2
+  {
+    int fd;
+
+    do
+    {
+      GNUNET_snprintf (target,
+                       slen,
+                       "%s.%u~",
+                       fil,
+                       num++);
+      fd = open (target,
+                 O_CREAT | O_EXCL,
+                 translate_unix_perms (GNUNET_DISK_PERM_USER_WRITE));
+    } while (-1 == fd);
+    if (0 != renameat2 (AT_FDCWD,
+                        fil,
+                        AT_FDCWD,
+                        target,
+                        RENAME_EXCHANGE))
+    {
+      GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR,
+                                "renameat2",
+                                fil);
+      GNUNET_break (0 ==
+                    close (fd));
+      return NULL;
+    }
+    GNUNET_break (0 ==
+                  close (fd));
+  }
+#else
   do
   {
     GNUNET_snprintf (target, slen, "%s.%u~", fil, num++);
   }
   while (0 == access (target, F_OK));
   if (0 != rename (fil, target))
+  {
     GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR, "rename", fil);
-  GNUNET_free (target);
+    return NULL;
+  }
+#endif
+  return target;
 }
 
 
@@ -633,34 +670,6 @@ GNUNET_DISK_file_read (const struct GNUNET_DISK_FileHandle *h,
 }
 
 
-ssize_t
-GNUNET_DISK_file_read_non_blocking (const struct GNUNET_DISK_FileHandle *h,
-                                    void *result,
-                                    size_t len)
-{
-  int flags;
-  ssize_t ret;
-
-  if (NULL == h)
-  {
-    errno = EINVAL;
-    return GNUNET_SYSERR;
-  }
-  /* set to non-blocking, read, then set back */
-  flags = fcntl (h->fd, F_GETFL);
-  if (0 == (flags & O_NONBLOCK))
-    (void) fcntl (h->fd, F_SETFL, flags | O_NONBLOCK);
-  ret = read (h->fd, result, len);
-  if (0 == (flags & O_NONBLOCK))
-  {
-    int eno = errno;
-    (void) fcntl (h->fd, F_SETFL, flags);
-    errno = eno;
-  }
-  return ret;
-}
-
-
 ssize_t
 GNUNET_DISK_fn_read (const char *fn,
                      void *result,
@@ -677,7 +686,8 @@ GNUNET_DISK_fn_read (const char *fn,
     return GNUNET_SYSERR;
   ret = GNUNET_DISK_file_read (fh, result, len);
   eno = errno;
-  GNUNET_assert (GNUNET_OK == GNUNET_DISK_file_close (fh));
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_DISK_file_close (fh));
   errno = eno;
   return ret;
 }
@@ -693,8 +703,9 @@ GNUNET_DISK_file_write (const struct GNUNET_DISK_FileHandle *h,
     errno = EINVAL;
     return GNUNET_SYSERR;
   }
-
-  return write (h->fd, buffer, n);
+  return write (h->fd,
+                buffer,
+                n);
 }
 
 
@@ -818,7 +829,6 @@ GNUNET_DISK_directory_scan (const char *dir_name,
 {
   DIR *dinfo;
   struct dirent *finfo;
-  struct stat istat;
   int count = 0;
   enum GNUNET_GenericReturnValue ret;
   char *name;
@@ -832,27 +842,10 @@ GNUNET_DISK_directory_scan (const char *dir_name,
     return GNUNET_SYSERR;
   while ((strlen (dname) > 0) && (dname[strlen (dname) - 1] == DIR_SEPARATOR))
     dname[strlen (dname) - 1] = '\0';
-  if (0 != stat (dname, &istat))
-  {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING, "stat", dname);
-    GNUNET_free (dname);
-    return GNUNET_SYSERR;
-  }
-  if (! S_ISDIR (istat.st_mode))
-  {
-    LOG (GNUNET_ERROR_TYPE_WARNING,
-         _ ("Expected `%s' to be a directory!\n"),
-         dir_name);
-    GNUNET_free (dname);
-    return GNUNET_SYSERR;
-  }
-  errno = 0;
   dinfo = opendir (dname);
-  if ((EACCES == errno) || (NULL == dinfo))
+  if (NULL == dinfo)
   {
     LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING, "opendir", dname);
-    if (NULL != dinfo)
-      closedir (dinfo);
     GNUNET_free (dname);
     return GNUNET_SYSERR;
   }
@@ -1017,7 +1010,6 @@ GNUNET_DISK_glob (const char *glob_pattern,
 
   if ( (NULL != strrchr (glob_pattern, '+')) ||
        (NULL != strrchr (glob_pattern, '[')) ||
-       (NULL != strrchr (glob_pattern, '+')) ||
        (NULL != strrchr (glob_pattern, '~')) )
   {
     LOG (GNUNET_ERROR_TYPE_ERROR,
@@ -1177,13 +1169,17 @@ GNUNET_DISK_file_copy (const char *src,
     pos += len;
   }
   GNUNET_free (buf);
-  GNUNET_DISK_file_close (in);
-  GNUNET_DISK_file_close (out);
+  GNUNET_break (GNUNET_OK ==
+                GNUNET_DISK_file_close (in));
+  GNUNET_break (GNUNET_OK ==
+                GNUNET_DISK_file_close (out));
   return GNUNET_OK;
-  FAIL:
+FAIL:
   GNUNET_free (buf);
-  GNUNET_DISK_file_close (in);
-  GNUNET_DISK_file_close (out);
+  GNUNET_break (GNUNET_OK ==
+                GNUNET_DISK_file_close (in));
+  GNUNET_break (GNUNET_OK ==
+                GNUNET_DISK_file_close (out));
   return GNUNET_SYSERR;
 }
 
@@ -1279,6 +1275,9 @@ GNUNET_DISK_file_open (const char *fn,
     }
   }
 
+  // Setting O_CLOEXEC after pipe() may introduce
+  // race conditions: https://bugs.gnunet.org/view.php?id=9311
+  // This is no problem if the platform supports pipe2
   fd = open (expfn,
              oflags
 #if O_CLOEXEC
@@ -1446,7 +1445,11 @@ GNUNET_DISK_pipe (enum GNUNET_DISK_PipeFlags pf)
 {
   int fd[2];
 
+#if HAVE_PIPE2 && O_CLOEXEC
+  if (-1 == pipe2 (fd, O_CLOEXEC))
+#else
   if (-1 == pipe (fd))
+#endif
   {
     int eno = errno;
 
@@ -1633,14 +1636,11 @@ GNUNET_DISK_pipe_handle (const struct GNUNET_DISK_PipeHandle *p,
 
 enum GNUNET_GenericReturnValue
 GNUNET_DISK_internal_file_handle_ (const struct GNUNET_DISK_FileHandle *fh,
-                                   void *dst,
-                                   size_t dst_len)
+                                   int *dst)
 {
   if (NULL == fh)
     return GNUNET_SYSERR;
-  if (dst_len < sizeof(int))
-    return GNUNET_SYSERR;
-  *((int *) dst) = fh->fd;
+  *dst = fh->fd;
   return GNUNET_OK;
 }
 
@@ -1660,14 +1660,22 @@ purge_cfg_dir (void *cls,
   char *tmpname;
 
   if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_filename (cfg, "PATHS", option, &tmpname))
+      GNUNET_CONFIGURATION_get_value_filename (cfg,
+                                               "PATHS",
+                                               option,
+                                               &tmpname))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR, "PATHS", option);
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "PATHS",
+                               option);
     return GNUNET_NO;
   }
-  if (GNUNET_SYSERR == GNUNET_DISK_directory_remove (tmpname))
+  if (GNUNET_SYSERR ==
+      GNUNET_DISK_directory_remove (tmpname))
   {
-    GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR, "remove", tmpname);
+    GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR,
+                              "remove",
+                              tmpname);
     GNUNET_free (tmpname);
     return GNUNET_OK;
   }
@@ -1677,11 +1685,13 @@ purge_cfg_dir (void *cls,
 
 
 void
-GNUNET_DISK_purge_cfg_dir (const char *cfg_filename,
+GNUNET_DISK_purge_cfg_dir (const struct GNUNET_OS_ProjectData *pd,
+                           const char *cfg_filename,
                            const char *option)
 {
   GNUNET_break (GNUNET_OK ==
-                GNUNET_CONFIGURATION_parse_and_run (cfg_filename,
+                GNUNET_CONFIGURATION_parse_and_run (pd,
+                                                    cfg_filename,
                                                     &purge_cfg_dir,
                                                     (void *) option));
 }
