@@ -179,7 +179,8 @@ editConfiguration (struct GNUNET_CONFIGURATION_Handle *cfg_,
   struct DiffsCBData diffsCB;
 
   initDiffsCBData (&diffsCB);
-  diffsCB.cfgDiffs = GNUNET_CONFIGURATION_create ();
+  diffsCB.cfgDiffs = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet
+                                                    ());
 
   switch (option)
   {
@@ -240,7 +241,7 @@ checkDiffs (struct GNUNET_CONFIGURATION_Handle *cfg_, int option)
 
   initDiffsCBData (&cbData);
 
-  cfg_new = GNUNET_CONFIGURATION_create ();
+  cfg_new = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet ());
   /* load defaults */
   GNUNET_assert (GNUNET_OK == GNUNET_CONFIGURATION_load (cfg_new, NULL));
 
@@ -258,8 +259,10 @@ checkDiffs (struct GNUNET_CONFIGURATION_Handle *cfg_, int option)
   GNUNET_CONFIGURATION_destroy (cfg_new);
 
   /* Compare the dumped configuration with modifications done */
-  cfg_new = GNUNET_CONFIGURATION_create ();
-  GNUNET_assert (GNUNET_OK == GNUNET_CONFIGURATION_parse (cfg_new, diffsFileName));
+  cfg_new = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet ());
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_CONFIGURATION_parse (cfg_new,
+                                             diffsFileName));
   if (0 != remove (diffsFileName))
     GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_WARNING, "remove",
                               diffsFileName);
@@ -302,6 +305,7 @@ testConfig ()
 {
   char *c;
   unsigned long long l;
+  float f;
 
   if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_string (cfg, "test", "b", &c))
     return 1;
@@ -344,6 +348,36 @@ testConfig ()
   }
   GNUNET_free (c);
 
+  // This only works if we have this locale.
+  if (NULL != setlocale (LC_NUMERIC, "de_DE"))
+  {
+    GNUNET_CONFIGURATION_set_value_string (cfg, "NUMBERS", "FLOAT_TEST_INVALID",
+                                           "1,23");
+    if (GNUNET_SYSERR !=
+        GNUNET_CONFIGURATION_get_value_float (cfg, "NUMBERS",
+                                              "FLOAT_TEST_INVALID", &f))
+    {
+      printf ("Parsed: %f\n", f);
+      GNUNET_break (0);
+      return 6;
+    }
+    GNUNET_CONFIGURATION_set_value_float (cfg, "NUMBERS", "FLOAT_TEST", 1.23);
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_string (cfg, "NUMBERS", "FLOAT_TEST", &c)
+        )
+    {
+      GNUNET_break (0);
+      return 6;
+    }
+    if (0 != strncmp (c, "1.23", 4))
+    {
+      printf ("%s\n", c);
+      GNUNET_free (c);
+      GNUNET_break (0);
+      return 7;
+    }
+    GNUNET_free (c);
+  }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_filename (cfg, "last", "test", &c))
   {
@@ -493,7 +527,7 @@ main (int argc, char *argv[])
   char *c;
 
   GNUNET_log_setup ("test_configuration", "WARNING", NULL);
-  cfg = GNUNET_CONFIGURATION_create ();
+  cfg = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet ());
   GNUNET_assert (cfg != NULL);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_parse (cfg, "test_configuration_data.conf"))
@@ -519,9 +553,10 @@ main (int argc, char *argv[])
   GNUNET_CONFIGURATION_destroy (cfg);
   GNUNET_assert (0 == unlink ("/tmp/gnunet-test.conf"));
 
-  cfg = GNUNET_CONFIGURATION_create ();
+  cfg = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet ());
   if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_load (cfg, "test_configuration_data.conf"))
+      GNUNET_CONFIGURATION_load (cfg,
+                                 "test_configuration_data.conf"))
   {
     GNUNET_break (0);
     GNUNET_CONFIGURATION_destroy (cfg);
@@ -546,8 +581,10 @@ main (int argc, char *argv[])
   GNUNET_CONFIGURATION_destroy (cfg);
 
   /* Testing configuration diffs */
-  cfg_default = GNUNET_CONFIGURATION_create ();
-  if (GNUNET_OK != GNUNET_CONFIGURATION_load (cfg_default, NULL))
+  cfg_default = GNUNET_CONFIGURATION_create (GNUNET_OS_project_data_gnunet ());
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_load (cfg_default,
+                                 NULL))
   {
     GNUNET_break (0);
     GNUNET_CONFIGURATION_destroy (cfg_default);
