@@ -556,6 +556,8 @@ postgres_plugin_get_closest (void *cls,
   return res;
 }
 
+void *
+libgnunet_plugin_datacache_postgres_init (void *cls);
 
 /**
  * Entry point for the plugin.
@@ -590,6 +592,8 @@ libgnunet_plugin_datacache_postgres_init (void *cls)
   return api;
 }
 
+void *
+libgnunet_plugin_datacache_postgres_done (void *cls);
 
 /**
  * Exit point from the plugin.
