@@ -33,6 +33,7 @@
 #include "gnunet_gnsrecord_json_lib.h"
 #include "microhttpd.h"
 #include <jansson.h>
+#include "namestore_plugin.h"
 
 /**
  * Namestore namespace
@@ -362,7 +363,7 @@ do_error (void *cls)
  * @param name the name of an identity (only one can be NULL)
  * @return EgoEntry or NULL if not found
  */
-struct EgoEntry *
+static struct EgoEntry *
 get_egoentry_namestore (struct RequestHandle *handle, char *name)
 {
   struct EgoEntry *ego_entry;
@@ -581,7 +582,7 @@ ns_get_lookup_cb (void *cls,
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_get (struct GNUNET_REST_RequestHandle *con_handle,
                const char *url,
                void *cls)
@@ -728,6 +729,7 @@ static void
 import_next_cb (void *cls, enum GNUNET_ErrorCode ec)
 {
   struct RequestHandle *handle = cls;
+  unsigned int remaining;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Import finished (%d)\n", ec);
@@ -738,7 +740,7 @@ import_next_cb (void *cls, enum GNUNET_ErrorCode ec)
     GNUNET_SCHEDULER_add_now (&do_error, handle);
     return;
   }
-  unsigned int remaining = handle->rd_set_count - handle->rd_set_pos;
+  remaining = handle->rd_set_count - handle->rd_set_pos;
   if (0 == remaining)
   {
     struct MHD_Response *resp;
@@ -757,24 +759,26 @@ import_next_cb (void *cls, enum GNUNET_ErrorCode ec)
     GNUNET_SCHEDULER_add_now (&cleanup_handle, handle);
     return;
   }
-  unsigned int sent_rds = 0;
-  // Find the smallest set of records we can send with our message size
-  // restriction of 16 bit
-  handle->ns_qe = GNUNET_NAMESTORE_records_store (handle->nc,
-                                                  handle->zone_pkey,
-                                                  remaining,
-                                                  &handle->ri[handle->
-                                                              rd_set_pos],
-                                                  &sent_rds,
-                                                  &import_next_cb,
-                                                  handle);
-  if ((NULL == handle->ns_qe) && (0 == sent_rds))
   {
-    handle->ec = GNUNET_EC_NAMESTORE_UNKNOWN;
-    GNUNET_SCHEDULER_add_now (&do_error, handle);
-    return;
+    unsigned int sent_rds = 0;
+    // Find the smallest set of records we can send with our message size
+    // restriction of 16 bit
+    handle->ns_qe = GNUNET_NAMESTORE_records_store (handle->nc,
+                                                    handle->zone_pkey,
+                                                    remaining,
+                                                    &handle->ri[handle->
+                                                                rd_set_pos],
+                                                    &sent_rds,
+                                                    &import_next_cb,
+                                                    handle);
+    if ((NULL == handle->ns_qe) && (0 == sent_rds))
+    {
+      handle->ec = GNUNET_EC_NAMESTORE_UNKNOWN;
+      GNUNET_SCHEDULER_add_now (&do_error, handle);
+      return;
+    }
+    handle->rd_set_pos += sent_rds;
   }
-  handle->rd_set_pos += sent_rds;
 }
 
 
@@ -783,6 +787,7 @@ bulk_tx_start (struct RequestHandle *handle)
 {
   json_t *data_js;
   json_error_t err;
+  char term_data[handle->rest_handle->data_size + 1];
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Transaction started...\n");
@@ -792,7 +797,6 @@ bulk_tx_start (struct RequestHandle *handle)
     GNUNET_SCHEDULER_add_now (&do_error, handle);
     return;
   }
-  char term_data[handle->rest_handle->data_size + 1];
   term_data[handle->rest_handle->data_size] = '\0';
   GNUNET_memcpy (term_data,
                  handle->rest_handle->data,
@@ -818,50 +822,53 @@ bulk_tx_start (struct RequestHandle *handle)
                               * sizeof (struct GNUNET_NAMESTORE_RecordInfo));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Got record set of size %d\n", handle->rd_set_count);
-  char *albl;
-  size_t index;
-  json_t *value;
-  json_array_foreach (data_js, index, value) {
-    {
-      struct GNUNET_GNSRECORD_Data *rd;
-      struct GNUNET_JSON_Specification gnsspec[] =
-      { GNUNET_GNSRECORD_JSON_spec_gnsrecord (&rd,
-                                              &handle->ri[index].a_rd_count,
-                                              &albl),
-        GNUNET_JSON_spec_end () };
-      if (GNUNET_OK != GNUNET_JSON_parse (value, gnsspec, NULL, NULL))
+  {
+    char *albl;
+    size_t index;
+    json_t *value;
+    json_array_foreach (data_js, index, value) {
       {
-        handle->ec = GNUNET_EC_NAMESTORE_RECORD_DATA_INVALID;
-        GNUNET_SCHEDULER_add_now (&do_error, handle);
-        json_decref (data_js);
-        return;
+        struct GNUNET_GNSRECORD_Data *rd;
+        struct GNUNET_JSON_Specification gnsspec[] =
+        { GNUNET_GNSRECORD_JSON_spec_gnsrecord (&rd,
+                                                &handle->ri[index].a_rd_count,
+                                                &albl),
+          GNUNET_JSON_spec_end () };
+        if (GNUNET_OK != GNUNET_JSON_parse (value, gnsspec, NULL, NULL))
+        {
+          handle->ec = GNUNET_EC_NAMESTORE_RECORD_DATA_INVALID;
+          GNUNET_SCHEDULER_add_now (&do_error, handle);
+          json_decref (data_js);
+          return;
+        }
+        handle->ri[index].a_rd = rd;
+        handle->ri[index].a_label = albl;
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                    "Parsed record set for name %s\n",
+                    handle->ri[index].a_label);
       }
-      handle->ri[index].a_rd = rd;
-      handle->ri[index].a_label = albl;
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                  "Parsed record set for name %s\n",
-                  handle->ri[index].a_label);
     }
   }
   // json_decref (data_js);
-
-  unsigned int sent_rds = 0;
-  // Find the smallest set of records we can send with our message size
-  // restriction of 16 bit
-  handle->ns_qe = GNUNET_NAMESTORE_records_store (handle->nc,
-                                                  handle->zone_pkey,
-                                                  handle->rd_set_count,
-                                                  handle->ri,
-                                                  &sent_rds,
-                                                  &import_next_cb,
-                                                  handle);
-  if ((NULL == handle->ns_qe) && (0 == sent_rds))
   {
-    handle->ec = GNUNET_EC_NAMESTORE_UNKNOWN;
-    GNUNET_SCHEDULER_add_now (&do_error, handle);
-    return;
+    unsigned int sent_rds = 0;
+    // Find the smallest set of records we can send with our message size
+    // restriction of 16 bit
+    handle->ns_qe = GNUNET_NAMESTORE_records_store (handle->nc,
+                                                    handle->zone_pkey,
+                                                    handle->rd_set_count,
+                                                    handle->ri,
+                                                    &sent_rds,
+                                                    &import_next_cb,
+                                                    handle);
+    if ((NULL == handle->ns_qe) && (0 == sent_rds))
+    {
+      handle->ec = GNUNET_EC_NAMESTORE_UNKNOWN;
+      GNUNET_SCHEDULER_add_now (&do_error, handle);
+      return;
+    }
+    handle->rd_set_pos += sent_rds;
   }
-  handle->rd_set_pos += sent_rds;
 }
 
 
@@ -872,7 +879,7 @@ bulk_tx_start (struct RequestHandle *handle)
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_import (struct GNUNET_REST_RequestHandle *con_handle,
                   const char *url,
                   void *cls)
@@ -921,7 +928,7 @@ namestore_import (struct GNUNET_REST_RequestHandle *con_handle,
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_add_or_update (struct GNUNET_REST_RequestHandle *con_handle,
                          const char *url,
                          void *cls)
@@ -1009,7 +1016,7 @@ namestore_add_or_update (struct GNUNET_REST_RequestHandle *con_handle,
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_update (struct GNUNET_REST_RequestHandle *con_handle,
                   const char *url,
                   void *cls)
@@ -1027,7 +1034,7 @@ namestore_update (struct GNUNET_REST_RequestHandle *con_handle,
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_add (struct GNUNET_REST_RequestHandle *con_handle,
                const char *url,
                void *cls)
@@ -1045,7 +1052,7 @@ namestore_add (struct GNUNET_REST_RequestHandle *con_handle,
  * @param url the url
  * @param cls the RequestHandle
  */
-void
+static void
 namestore_delete (struct GNUNET_REST_RequestHandle *con_handle,
                   const char *url,
                   void *cls)
